SystemOrganization addCategory: #'Alt-Browser'!
SystemOrganization addCategory: #'Alt-Browser-Commands'!
SystemOrganization addCategory: #'Alt-Browser-Tests'!

ComposableModel subclass: #AltBrowser
	instanceVariableNames: 'treeModel methodModel textModel'
	classVariableNames: 'announcer rootItems'
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltBrowser commentStamp: '<historical>' prior: 0!
An AltBrowser is the main application.

Instance Variables
	methodModel:		<A method model>
	treeModel:		<An ABPackageList>

methodModel
	- handle the display of methods.

treeModel
	- handle navigation between packages/category/classes/protocols/methods
!

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/28/2012 11:17'!
aCategoryRemoved: anEvent
	"Ok, a class category was removed."

	| node r |
	node := self rootItems contents
		detect: [ :e | (r := e findClassCategory: anEvent categorySelector) notNil ] ifNone: [^self].
	r parentNode contents remove: r.
	announcer announce: (AltNodeRemoved node: r)! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/29/2012 10:37'!
aClassAdded: anEvent
	"Ok, a class was added. Find the package it belongs to (or none)."

	| node r package cat |
	package := RPackageOrganizer default globalPackageOf: anEvent classAdded.
	self assert: package notNil.
	node := self rootItems contents detect: [ :e | (r := e findPackage: package) notNil ] ifNone: [ ^ self ].
	self rootItems contents detect: [ :e | (cat := e findClassCategory: anEvent classCategory) notNil ] ifNone: [ nil ].
	cat notNil
		ifTrue: [ 
			| classNode |
			cat contentsNotNil
				ifTrue: [ 
					classNode := cat addChild: anEvent classAdded.
					announcer announce: (AltNodeAdded node: classNode) ] ]
		ifFalse: [ 
			r contentsNotNil
				ifTrue: [ 
					| classNode |
					classNode := r addChild: anEvent classAdded.
					announcer announce: (AltNodeAdded node: classNode) ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/30/2012 13:24'!
aClassRemoved: anEvent
	"Ok, a class was removed. TODO: if this class has extensions, it should be removed from more than one place."

	| node r |
	node := self rootItems contents
		detect: [ :e | (r := e findClass: anEvent removedClass) notNil ] ifNone: [^self].
	r parentNode contents remove: r.
	announcer announce: (AltNodeRemoved node: r)! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/30/2012 13:33'!
aClassReorganized: anEvent
	"Ok, a class was reorganised (a new protocol, for example). It seems I need to rebuilt all nodes on that class."

	| node r o |
	o := OrderedCollection new.
	node := self rootItems contents
		do: [ :e | 
			(r := e findAllClass: anEvent classReorganized) notNil
				ifTrue: [ o addAll: r ] ].
	o
		do: [ :each | 
			each rebuildContents.
			announcer announce: (AltNodeRebuilt node: each) ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/28/2012 23:25'!
aMCWorkingCopyCreated: anEvent
	"Only case where we don't use a #findXXX: on the root items."
	
	"TODO: change to take in account the RPackage change."

	| package cat node |
	package := RPackageOrganizer default packageNamed: anEvent workingCopy packageName.
	cat := package name copyUpTo: $-.
	node := self rootItems contents detect: [ :e | e item key = cat ] ifNone: [ nil ].
	node notNil
		ifTrue: [ 
			node item value
				detect: [ :e | e == package ]
				ifNone: [ 
					| packageNode |
					node item value add: package.
					packageNode :=  node addChild: package.
					packageNode
						ifNotNil: [ announcer announce: (AltNodeAdded node: packageNode) ] ] ]
		ifFalse: [ 
			node := ABPackageCategoryNode with: cat -> (OrderedCollection with: package).
			self rootItems contents add: node.
			 announcer announce: (AltNodeAdded node: node)  ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/28/2012 23:27'!
aMCWorkingCopyDeleted: anEvent
	"Ok. Now, empty package categories are not removed automagically."

	| package cat node r |
	node := self rootItems contents
		detect: [ :e | (r := e findPackage: (RPackageOrganizer default packageNamed: anEvent workingCopy packageName)) notNil ] ifNone: [^ self].
	r parentNode removeChildItem: r.
	announcer announce: (AltNodeRemoved node: r) ! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/30/2012 16:34'!
aMethodAdded: anEvent
	"Ok, a method was added. Does not mean modify. Works."

	| mCNodes |
	self rootItems contents
		do: [ :root | 
			| res |
			res := root findAllMethodParents: anEvent selector forClass: anEvent methodClass.
			res notNil
				ifTrue: [ 
					mCNodes isNil
						ifTrue: [ mCNodes := res ]
						ifFalse: [ mCNodes addAll: res ] ] ].
	mCNodes isNil
		ifTrue: [ ^ self ].
	mCNodes
		do: [ :each | 
			each contentsNotNil
				ifTrue: [ 
					| s |
					s := each addChild: anEvent selector.
					announcer announce: (AltNodeAdded node: s) ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/28/2012 15:28'!
aMethodRemoved: anEvent
	"Ok, a method was removed. Don't forget the method may be in multiple places."
	
	"Change this code; search for places where the method is visible (extensions and protocols)."

	| mCNodes |
	self rootItems contents
		do: [ :root | 
			| res |
			res := root findAllMethodParents: anEvent selector forClass: anEvent methodClass.
			res notNil
				ifTrue: [ 
					mCNodes isNil
						ifTrue: [ mCNodes := res ]
						ifFalse: [ mCNodes addAll: res ] ] ].
	mCNodes isNil ifTrue: [^ self].
	mCNodes
		do: [ :each | 
			each contentsNotNil
				ifTrue: [ 
					| s |
					each contents remove: (s := each findMethod: anEvent selector).
					announcer announce: (AltNodeRemoved node: s) ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 8/28/2012 13:25'!
aNodeToRebuilt: r
	announcer announce: (AltNodeRebuilt node: r)! !

!AltBrowser class methodsFor: 'accessing' stamp: 'ThierryGoubier 8/28/2012 14:03'!
announcer
	announcer isNil
		ifTrue: [ announcer := Announcer new ].
	^announcer! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 8/28/2012 21:49'!
defaultPackageCategoriesNames
	"As it says: categories for packages who don't use the - convention. But also the place where we may start organizing nicely the system to have a better structure. Do a reset of the AltBrowser to see the changes in the opened browsers."
	
	"AltBrowser reset."
	
	^ #('System'
		#('GUI' 'Graphics' 'Morphic' 'Balloon' 'FreeType' 'Polymorph' 'Spec' 'UIManager' 'UI' 'GroupManagerUI' 'Growl' 'Keymapping' 'FamFam')
		#('Core' 'Kernel' 'Collections' 'Text' 'Compiler' 'EmergencyEvaluator' 'Regex' 'Multilingual' 'Announcements' 'Transcript' 'Traits')
		#('Development' 'AST' 'Refactoring' 'Tools' 'Nautilus' 'NonInteractiveTranscript' 'SUnit' 'Tests' 'Shout' 'SmaCC' 'NECompletion' 'NOCompletion')
		#('Networking' 'Network' 'Zodiac' 'Zinc' 'Compression')
		#('Packages' 'PackageInfo' 'Fuel' 'Monticello' 'CodeImport' 'Gofer' 'Configuration' 'RPackage' 'Ring' 'CI' 'GroupManager')
		#('2.0' 'Compatibility' 'Deprecated' 'RecentSubmissions' 'FixUnderscores' 'ScriptLoader20')
		
	)! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 8/1/2012 11:05'!
defaultSpec
	<spec>
	
	^ {#ComposableSpec.
			#add:. {
				{#model. #treeModel}.
				#layout:.
				{#FrameLayout. #fractions:offsets:. (0 @ 0 corner: 0.3 @ 1). (0 @ 0 corner: 0 @ 0)}
			}.
			#add:. {
				{#model. #textModel}.
				#layout:.
				{#FrameLayout. #fractions:offsets:. (0.3 @ 0 corner: 1 @ 1). (0 @ 0 corner: 0 @ 0)}
			}
		}! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/26/2012 10:42'!
example
	"AltBrowser example"
	
	AltBrowser new openWithSpec! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 8/1/2012 10:39'!
getItemsFor: aCategoryDefinition from: aListOfPackages
	"Remove elements from aListOfPackages to bring them accordingly to a CategoryDefinition. Returns the top-level object. If this definition is an array, go recursively through it."

	aCategoryDefinition isArray
		ifTrue: [
			| i |
			i := aCategoryDefinition first -> OrderedCollection new.
			aCategoryDefinition allButFirst do: [:each | i value add: (self getItemsFor: each from: aListOfPackages)].
			^ i]
		ifFalse: [
			| i |
			i := aCategoryDefinition -> OrderedCollection new.
			(aListOfPackages
				select:
					[ :e |
					e packageName size >= aCategoryDefinition size
						and: [ (e packageName copyFrom: 1 to: aCategoryDefinition size) = aCategoryDefinition ] ])
				do: [ :e | 
					aListOfPackages remove: e.
					i value add: e ].
			^ i ]! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 8/29/2012 17:29'!
getRootItems
	"This is where I should rework the organisation. Use broad classes to reduce the number of top-level items. To be improved. Does not catch classes which have no packages."

	| l o |
	l := RPackageOrganizer default packages asOrderedCollection.
	o := OrderedCollection new.
	self defaultPackageCategoriesNames
		do: [ :w | 
			| i |
			i := self getItemsFor: w from: l.
			o add: i ].
	[ l notEmpty ]
		whileTrue: [ 
			| p w i |
			p := l first.
			w := p packageName copyUpTo: $-.
			i := w -> (OrderedCollection with: p).
			o add: i.
			l remove: p.
			(l select: [ :e | (e packageName copyUpTo: $-) = w ])
				do: [ :e | 
					l remove: e.
					i value add: e ] ].
	^ (SortedCollection sortBlock: [:a :b | a asString < b asString])
		addAll: (o collect: [ :e | ABPackageCategoryNode with: e ]);
		asOrderedCollection! !

!AltBrowser class methodsFor: 'initialization' stamp: 'ThierryGoubier 8/28/2012 13:33'!
initialize
	self registerOnSystemAnnouncements.
	announcer := Announcer new.
	rootItems := nil! !

!AltBrowser class methodsFor: 'user interface' stamp: 'ThierryGoubier 8/29/2012 14:54'!
menuCommandOn: aBuilder 
	<worldMenu>
	
	(aBuilder item: #'Alt Browser')
			parent: #Tools;
			action: [ self open ];
			icon: UITheme current smallSystemBrowserIcon;
			help: 'An Alternative System Browser'.! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/30/2012 13:42'!
open
	"AltBrowser example"
	
	AltBrowser new openWithSpec! !

!AltBrowser class methodsFor: 'initialization' stamp: 'ThierryGoubier 8/30/2012 10:17'!
registerOnSystemAnnouncements
	"Global registration to update the list."

	"self registerOnSystemAnnouncements."

	SystemAnnouncer uniqueInstance unsubscribe: self.

	SystemAnnouncer uniqueInstance weak
		on: MCWorkingCopyCreated send: #aMCWorkingCopyCreated: to: self;
		on: MCWorkingCopyDeleted send: #aMCWorkingCopyDeleted: to: self;
		on: CategoryRemoved send: #aCategoryRemoved: to: self;
		on: MethodAdded send: #aMethodAdded: to: self;
		on: MethodRemoved send: #aMethodRemoved: to: self;
		on: ClassAdded send: #aClassAdded: to: self;
		on: ClassRemoved send: #aClassRemoved: to: self;
		on: ClassReorganized send: #aClassReorganized: to: self! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 8/28/2012 10:59'!
reset
	"reset the rootItems. Should propagate to all instances as well."
	
	"AltBrowser reset"

	self rootItems contents: self getRootItems! !

!AltBrowser class methodsFor: 'accessing' stamp: 'ThierryGoubier 8/28/2012 10:58'!
rootItems
	rootItems isNil
		ifTrue: [ rootItems := NewValueHolder contents: self getRootItems ].
	^ rootItems! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/15/2012 22:44'!
title
	^ 'Alt Browser'! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 7/26/2012 16:23'!
updateAll
	"Entry point for updating the whole tree if needed."
	
	self reset! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 8/28/2012 14:07'!
addNode: aNodeAdded
	"Announcement : A node has been added."

	treeModel rootsAdd: aNodeAdded node! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 7/26/2012 21:19'!
addPackageCategory: aNode
	"A new package category has been added. Update the tree by applying it."

	treeModel rootsAdd: (ABTreeNodeModel with: aNode model: aNode)! !

!AltBrowser methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:22'!
buildMenu: aMenu on: aTarget
	treeModel selectedItem notNil
		ifTrue: [ treeModel selectedItem buildTextMenu: aMenu on: aTarget with: self ].
	^ aMenu! !

!AltBrowser methodsFor: 'commands' stamp: 'ThierryGoubier 7/31/2012 15:57'!
buildTreeMenu: aMenu on: aTarget
	"Ask the selection for builidng a menu, otherwise create a default one."
	
	treeModel selectedItem notNil
		ifTrue: [ treeModel selectedItem buildTreeMenu: aMenu on: aTarget with: self ]
		ifFalse: [ 
			| t |
			t := false.
			(Pragma allNamed: #treeAreaCommand from: self class to: AltBrowser)
				do: [ :e | 
					t
						ifTrue: [ aMenu addLine ]
						ifFalse: [ t := true ].
					(self perform: e selector) do: [ :c | (c on: aTarget for: self) addItemToMenu: aMenu ] ] ].
	^ aMenu! !

!AltBrowser methodsFor: 'menu' stamp: 'ThierryGoubier 7/31/2012 15:57'!
defaultCommands
	<treeAreaCommand>
	^ {ABCreatePackageCategoryCommand}! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 8/30/2012 16:21'!
findNodeForItem: anItem
	| result |
	treeModel rootItems detect:
		[:rootItem |
		(result := rootItem findItem: anItem) notNil ].
	^ result! !

!AltBrowser methodsFor: 'private' stamp: 'ThierryGoubier 8/28/2012 13:33'!
getRootItems
	^ self class rootItems contents collect: [ :e | ABTreeNodeModel with: e model: e ]! !

!AltBrowser methodsFor: 'protocol' stamp: 'ThierryGoubier 8/7/2012 20:31'!
initialExtent
	^ 700 @ 500! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 8/29/2012 15:44'!
initializePresenter
	"The models are created, it means I can register on events."
	
	treeModel roots: self getRootItems.
	treeModel childrenBlock: [ :it | it contents ].
	treeModel displayBlock: [ :it | it asString ].
	treeModel
		whenSelectedItemChanged: [ :sel |
			self updateTextKeymap.
			sel isNil
				ifTrue: [ 
					textModel text: ''.
					textModel behavior: nil.
					textModel aboutToStyle: false ]
				ifFalse: [ 
					textModel text: sel sourceCode.
					textModel behavior: sel sourceClass.
					textModel aboutToStyle: sel isStyled ] ].
	textModel menu: [ :m :t | self buildMenu: m on: t ].
	treeModel menu: [:m :t | self buildTreeMenu: m on: t].
	self class rootItems whenChangedSend: #reset to: self.
	self class announcer weak on: AltNodeRebuilt send: #rebuiltNode: to: self.
	self class announcer weak on: AltNodeRemoved send: #removeNode: to: self.
	self class announcer weak on: AltNodeAdded send: #addNode: to: self! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 7/17/2012 22:43'!
initializeWidgets
	self instantiateModels: #(#treeModel #ABTreeModel #textModel #ABTextModel).
	textModel aboutToStyle: true! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 8/29/2012 11:02'!
rebuiltNode: aNodeRebuilt
	"Announcement : a node has been rebuilt (debugging)."
	
	treeModel rebuildNode: aNodeRebuilt node! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 8/29/2012 22:16'!
release
	"There is something about releasing models."
	
	treeModel ifNotNil: [treeModel initialize. treeModel := nil].
	textModel ifNotNil: [textModel initialize. textModel := nil].
	self class announcer unsubscribe: self.
	super release.
! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 8/28/2012 14:08'!
removeNode: aNodeRemoved
	"Announcement : A node in the tree has been removed. Update the tree."

	| aNode |
	aNode := aNodeRemoved node.
	(self selectedItem notNil and: [self selectedItem item == aNode])
		ifTrue: [ self selectedItem: nil ].
	treeModel rootsRemove: aNode! !

!AltBrowser methodsFor: 'private' stamp: 'ThierryGoubier 8/1/2012 22:17'!
reset
	treeModel isNil
		ifFalse: [treeModel roots: self getRootItems.
			textModel pendingText: '']! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/26/2012 10:45'!
selectedItem
	^ treeModel selectedItem! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/27/2012 16:38'!
selectedItem: anItem
	treeModel setSelectedItem: anItem! !

!AltBrowser methodsFor: 'shortcuts' stamp: 'ThierryGoubier 8/5/2012 22:58'!
targetShortcutMorph
	"Get the real, lowest level morph we need to touch the key dispatch for. What a mess!!"

	^ self textModel widget submorphs first submorphs first! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 21:56'!
taskbarIcon
	^ UITheme current smallSystemBrowserIcon! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/16/2012 00:35'!
textModel
	^textModel! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/15/2012 22:41'!
treeModel
	^treeModel! !

!AltBrowser methodsFor: 'shortcuts' stamp: 'ThierryGoubier 8/5/2012 22:59'!
updateTextKeymap
	"Update the text keymap."

	| keyMorph |
	keyMorph := self targetShortcutMorph.
	(keyMorph kmDispatcher isKindOf: AltKMDispatcher)
		ifFalse: [ keyMorph setProperty: #kmDispatcher toValue: (AltKMDispatcher target: keyMorph) ]
		ifTrue: [ keyMorph kmDispatcher reset ].
	self selectedItem notNil
		ifTrue: [ self selectedItem item buildTextShortcutsOn: keyMorph with: self ]
		
		"	| c |
"	"First remove the category for our widget, this way removing the keymap itself."	"	self textModel widget detachAllKeymapCategories.
	KMRepository default categoryHolders removeKey: self textModel widget ifAbsent: [#()].
"	"Then remove all shortcuts from the KMRepository."	"	KMRepository default categories removeKey: ('AltBrowser', self identityHash printString) asSymbol ifAbsent: [nil].
"	"If the selected item is not nil, rebuild a keymap."	"	self selectedItem notNil
		ifTrue: [ | keymapBuilder |
			keymapBuilder := self selectedItem item buildTextShortcutsOn: self textModel with: self.
			self textModel widget attachKeymapCategory: ('AltBrowser', self identityHash printString) asSymbol targetting: self]
	"! !

!WindowModel methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 8/1/2012 21:55'!
taskbarIcon
	^ self model taskbarIcon! !

Model subclass: #ABAbstractNode
	instanceVariableNames: 'item contents parentNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABAbstractNode commentStamp: '<historical>' prior: 0!
An ABAbstractNode is the abstract type of all nodes used by the ABPackageList.!

!ABAbstractNode class methodsFor: 'instance creation' stamp: 'ThierryGoubier 7/19/2012 22:36'!
with: anItem
	^ self new item: anItem! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/26/2012 14:15'!
accept: aText
	"When the text editor associated has accepted, do that..."

	"That is, nothing by default."

	! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/30/2012 17:22'!
addChild: anItem
	"Add a child, but do it according to it's position."

	| d |
	d := self childrenItems indexOf: anItem.
	^ contents add: (self childNodeFromItem: anItem) beforeIndex: d! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/30/2012 00:00'!
addChildItem: anItem
	"Add a child item. Remember to effectively add both at the item level if necessary and at the contents level."

	| d |
	self childrenItems isEmpty ifTrue: [^self contents add: anItem].
	d := self childrenItems indexOf: anItem item.
	^ contents add: anItem beforeIndex: d! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 21:38'!
addChildItem: anItem beforeIndex: index
	"Add a child, but do it according to it's position."

	anItem parentNode: self.
	^ contents add: anItem beforeIndex: index! !

!ABAbstractNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/19/2012 22:43'!
asString
	^ item asString! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/5/2012 23:05'!
buildTextMenu: aMenu on: aTarget with: aRequestor
	| t |
	t := false.
	(Pragma allNamed: #textAreaCommand from: self class to: ABAbstractNode)
		do: [ :e | 
			t
				ifTrue: [ aMenu addLine ]
				ifFalse: [ t := true ].
			(self perform: e selector) do: [ :c | (c on: aTarget for: aRequestor) addItemToMenu: aMenu ] ]! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/5/2012 23:05'!
buildTextShortcutsOn: aKMDispatcher with: aRequestor
	"This is an attempt at handling shortcuts... Which works, with the help of a custom KMDispatcher."

	(Pragma allNamed: #textAreaCommand from: self class to: ABAbstractNode)
		do: [ :e | (self perform: e selector) do: [ :c | (c on: aRequestor textModel for: aRequestor) buildShortcut: aKMDispatcher ] ]! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/30/2012 16:46'!
buildTreeMenu: aMenu on: aTarget with: aRequestor
	| t |
	t := false.
	(Pragma allNamed: #treeAreaCommand from: self class to: ABAbstractNode)
		do: [ :e | 
			t
				ifTrue: [ aMenu addLine ]
				ifFalse: [ t := true ].
			(self perform: e selector) do: [ :c | (c on: aTarget for: aRequestor) addItemToMenu: aMenu ] ]! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/19/2012 22:32'!
childNodeClassFromItem: anItem
	^ self class! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/27/2012 21:34'!
childNodeFromItem: anItem
	^ ((self childNodeClassFromItem: anItem) with: anItem)
		parentNode: self;
		yourself! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/19/2012 22:34'!
childrenItems
	^ #()! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:18'!
clipboardCommands
	<textAreaCommand>
	^ {ABCutTextCommand.
	ABCopyTextCommand.
	ABPasteTextCommand}! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:32'!
contents
	contents isNil
		ifTrue: [ contents := self childrenItems collect: [ :i | self childNodeFromItem: i ] ].
	^ contents! !

!ABAbstractNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/27/2012 21:47'!
contentsNotNil
	^ contents notNil! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 8/5/2012 09:07'!
debugTreeCommands
	<treeAreaCommand>
	^ {ABExploreItemCommand. ABRebuildItemCommand }! !

!ABAbstractNode methodsFor: 'drag and drop' stamp: 'ThierryGoubier 8/5/2012 21:15'!
dropItem: anItem
	"By default, do nothing."! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:18'!
executeCommands
	<textAreaCommand>
	^ {ABDoItTextCommand.
	ABPrintItTextCommand.
	ABInspectItTextCommand.
	ABExploreItTextCommand.
	ABDebugItTextCommand}! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/30/2012 13:34'!
findAllClass: aClass
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	| o |
	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findAllClass: aClass) notNil
						ifTrue: [
							o isNil
								ifTrue: [ o := res ]
								ifFalse: [ o addAll: res ] ] ] ].
	^ o! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/28/2012 15:18'!
findAllMethodParents: aMethodSelector forClass: aClass
	"Return all nodes if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	| o |
	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findAllMethodParents: aMethodSelector forClass: aClass) notNil
						ifTrue: [ 
							o isNil
								ifTrue: [ o := res ]
								ifFalse: [ o addAll: res ] ] ] ].
	^ o! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 11:34'!
findClass: aClass
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findClass: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 10:24'!
findClassCategory: aCat
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findClassCategory: aCat) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 14:21'!
findClassComment: aClass
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findClassComment: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/28/2012 15:16'!
findMethod: aSelector
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findMethod: aSelector) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 13:41'!
findMethodCategory: aMethodCategory
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findMethodCategory: aMethodCategory) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 21:15'!
findPackage: aMCWorkingCopy
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	contents isNil
		ifFalse: [ 
			| res |
			contents
				do: [ :each | 
					(res := each findPackage: aMCWorkingCopy) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/30/2012 14:01'!
indexOfChild: anItem
	^ self childrenItems indexOf: anItem! !

!ABAbstractNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/19/2012 23:20'!
isStyled
	"Should return true if the text is to be styled (default smalltalk code)."
	
	^ false! !

!ABAbstractNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 14:44'!
isTreeNodeModel
	^ false! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
item
	^ item! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
item: anObject
	item := anObject! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
parentNode
	^ parentNode! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
parentNode: anObject
	parentNode := anObject! !

!ABAbstractNode methodsFor: 'printing' stamp: 'ThierryGoubier 8/30/2012 07:04'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self asString.
	aStream nextPut: $)! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 08:50'!
rebuildContents
	"Force a rebuild of the contents, to either : change the layout of the children items, or just because the contents has become out of sync with the children items."
	
	contents := nil! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 21:24'!
removeChildItem: anItem
	"Remove a child item. Some subclasses may do things with the item, or translate this into an operation."

	self contents remove: anItem! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:19'!
saveCommands
	<textAreaCommand>
	^ {ABAcceptTextCommand.
	ABCancelTextCommand}! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 8/7/2012 22:54'!
searchTreeCommands
	<treeAreaCommand>
	^ {ABFindClassCommand }! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/19/2012 23:20'!
sourceClass
	"Should return a class if appropriate. The class is to provide the context for compilation, evaluation or styling."

	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/19/2012 23:19'!
sourceCode
	"Should return a source code or a description if appropriate."
	
	^ ''! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:19'!
undoCommands
	<textAreaCommand>
	^ {ABUndoTextCommand.
	ABRedoTextCommand}! !

ABAbstractNode subclass: #ABClassCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassCategoryNode commentStamp: '<historical>' prior: 0!
An ABClassCategoryNode is for class categories.
!

!ABClassCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/31/2012 22:28'!
childNodeClassFromItem: anItem
	(anItem isBehavior or: [ anItem isTrait ])
		ifTrue: [ ^ ABClassNode ]
		ifFalse: [ ^ self class ]! !

!ABClassCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:08'!
childrenItems
	"Extract from the organisation the various classes. Ok. Our upper level should try to built a hierarchy with the category name, if applicable (via various upTo: $-). This code is ready for that (introduce an additional hierarchy)."

	| c o |
	c := Smalltalk organization categoriesMatching: self item.
	c size > 2
		ifTrue: [ ^ c ]
		ifFalse: [ 
			o := OrderedCollection new.
			c do: [ :e | (Smalltalk organization classesInCategory: e) do: [ :class | o add: class ] ].
			^ o ]! !

!ABClassCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 10:25'!
findClassCategory: aCat
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	item == aCat
		ifTrue: [ ^ self ]
		ifFalse: [ ^ nil ]! !

ABAbstractNode subclass: #ABClassCommentNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassCommentNode commentStamp: '<historical>' prior: 0!
An ABClassCommentNode is for a class comment, if any. No children.
!

!ABClassCommentNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/26/2012 14:23'!
accept: aText
	"This should change the class comment."

	self item comment: aText! !

!ABClassCommentNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/20/2012 07:10'!
asString
	^ self item hasComment
		ifTrue: [ 'comment' asText addAttribute: TextEmphasis italic ]
		ifFalse: [ 'comment' ]! !

!ABClassCommentNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 14:30'!
findClassComment: aClass
	^ item == aClass
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABClassCommentNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/16/2012 20:36'!
sourceCode
	^ self item comment! !

ABAbstractNode subclass: #ABClassNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassNode commentStamp: '<historical>' prior: 0!
An ABClassNode represent a class!

ABClassNode subclass: #ABClassExtensionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassExtensionNode commentStamp: '<historical>' prior: 0!
An ABClassExtensionNode is used when browsing the extension of a package. Restrict itself to the methods of the extension of the parent package.!

!ABClassExtensionNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/17/2012 13:37'!
asString
	^ self item asString! !

!ABClassExtensionNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:39'!
childNodeClassFromItem: anItem
	^ ABMethodNode! !

!ABClassExtensionNode methodsFor: 'private' stamp: 'ThierryGoubier 8/27/2012 00:19'!
childrenItems
	^ ((self parentNode item extensionMethodsForClass: self item) collect: [ :e | e selector ])
		asSortedCollection! !

!ABClassExtensionNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 8/28/2012 15:21'!
findAllMethodParents: aMethodSelector forClass: aClass
	"Ask the package to know if this method is an extension."
	
	^ (item == aClass and: [ parentNode item includesExtensionSelector: aMethodSelector ofClass:  item])
		ifTrue: [ OrderedCollection with: self ]
		ifFalse: [ nil ]! !

!ABClassExtensionNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/28/2012 17:28'!
methodCategory
	"Still working in fact."
	
	^ parentNode item methodCategoryPrefix! !

!ABClassExtensionNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/31/2012 21:58'!
sourceClass
	^ self item! !

!ABClassExtensionNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/31/2012 22:08'!
sourceCode
	^ 'message selector and argument names
	"comment stating purpose of message"

	| temporary variable names |
	statements'! !

!ABClassNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/26/2012 14:31'!
accept: aText
	^ [ 
	UndefinedObject evaluatorClass new
		evaluate: (ReadStream on: aText)
		in: nil
		to: nil
		notifying: nil
		ifFail: [ ^ nil ]
		logged: true ]
		on: OutOfScopeNotification
		do: [ :ex | ex resume: true ]! !

!ABClassNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/17/2012 10:33'!
asString
	^ self item isMeta
		ifTrue: [ '- Class' ]
		ifFalse: [ self item asString ]! !

!ABClassNode methodsFor: 'private' stamp: 'ThierryGoubier 7/16/2012 20:07'!
childNodeClassFromItem: anItem
	anItem == self item classSide
		ifTrue: [ ^ self class ].
	anItem == self item
		ifTrue: [ ^ ABClassCommentNode ].
	^ ABMethodCategoryNode! !

!ABClassNode methodsFor: 'private' stamp: 'ThierryGoubier 7/16/2012 20:06'!
childrenItems
	^ self item isMeta
		ifFalse: [ 
			{(self item classSide).
			(self item)} , self item organization categories ]
		ifTrue: [ self item organization categories ]! !

!ABClassNode methodsFor: 'private' stamp: 'ThierryGoubier 7/16/2012 13:56'!
classSide
	^ self item isMeta
		ifFalse: [ {(self item classSide)} ]
		ifTrue: [ {} ]! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/30/2012 10:48'!
findAllClass: aClass
	^ self item == aClass
		ifTrue: [ OrderedCollection with: self ]
		ifFalse: [ nil ]! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 11:35'!
findClass: aClass
	^ self item == aClass
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 22:31'!
findMethodCategory: aMethodCategory
	"Interesting bug: class and instance side may have the exact same method category."
	
	^ self item isMeta
		ifTrue: [ nil ]
		ifFalse: [ super findMethodCategory: aMethodCategory ]! !

!ABClassNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/19/2012 23:23'!
isStyled
	^ true! !

!ABClassNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:54'!
privateClass
	^ self item! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/16/2012 14:18'!
sourceCode
	^ self item definition! !

ABAbstractNode subclass: #ABMethodCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABMethodCategoryNode commentStamp: '<historical>' prior: 0!
For a method category. No need for an -- all ---, the tree list allow for opening everything if needed. The 'as yet unclassified' method category (protocol) is managed on the fly (and exist in the system) !

!ABMethodCategoryNode methodsFor: 'actions' stamp: 'ThierryGoubier 8/30/2012 16:33'!
accept: aText
	"Compile the method."

	self sourceClass compile: aText classified: self methodCategory notifying: nil! !

!ABMethodCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/16/2012 00:18'!
childNodeClassFromItem: anItem
	^ ABMethodNode! !

!ABMethodCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/27/2012 16:25'!
childrenItems
	^ (self parentNode item methodsInCategory: self item) asSortedCollection! !

!ABMethodCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 8/29/2012 11:21'!
findAllMethodParents: aMethodSelector forClass: aClass
	^ (parentNode item == aClass and: [ (parentNode item whichCategoryIncludesSelector: aMethodSelector) = item ])
		ifTrue: [ OrderedCollection with: self ]
		ifFalse: [ nil ]! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 14:27'!
findMethodCategory: aMethodCategory
	^ aMethodCategory = item
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABMethodCategoryNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 22:46'!
isStyled
	^ true! !

!ABMethodCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 22:00'!
methodCategory
	^ item! !

!ABMethodCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:54'!
privateClass
	^ self parentNode privateClass! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/17/2012 13:53'!
sourceClass
	^ self parentNode item! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/31/2012 22:09'!
sourceCode
	^ 'message selector and argument names
	"comment stating purpose of message"

	| temporary variable names |
	statements'! !

!ABMethodCategoryNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 8/30/2012 16:15'!
treeAreaCommands
	<treeAreaCommand>
	^ {ABAddProtocolCommand. ABRemoveProtocolCommand }! !

ABAbstractNode subclass: #ABMethodNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABMethodNode commentStamp: '<historical>' prior: 0!
Represent a method. Should be used to get the method source, selector and whatever.!

!ABMethodNode methodsFor: 'actions' stamp: 'ThierryGoubier 7/31/2012 21:55'!
accept: aText
	"Compile the method."

	self sourceClass compile: aText classified: self parentNode methodCategory notifying: nil! !

!ABMethodNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 13:42'!
findMethod: anItem
	^ item == anItem
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABMethodNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/19/2012 23:22'!
isStyled
	^ true! !

!ABMethodNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/17/2012 13:55'!
sourceClass
	"Get the class through the parent."

	^ (self parentNode privateClass compiledMethodAt: self item) methodClass! !

!ABMethodNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/17/2012 13:55'!
sourceCode
	"Get the source code through the parent."

	^ (self parentNode privateClass compiledMethodAt: self item) sourceCode! !

ABAbstractNode subclass: #ABPackageCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageCategoryNode commentStamp: '<historical>' prior: 0!
An ABPackageCategoryNode represent a category of packages.

item contains an association "package category name" -> "list of packages.".

!

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 09:44'!
addChild: anItem
	"Add a child, but do it according to it's position."

	| d |
	item value add: anItem.
	d := self childrenItems indexOf: anItem.
	contents isNil ifTrue: [^nil] ifFalse: [
	^ contents add: (self childNodeFromItem: anItem) beforeIndex: d]! !

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 10:21'!
addChildItem: anItem
	"Add a child, but do it according to it's position."

	| d |
	self contents.	"Force contents creation."
	item value add: anItem item.
	anItem parentNode: self.
	^ contents add: anItem! !

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/30/2012 06:59'!
addChildItem: anItem beforeIndex: anIndex
	"Add a child, but do it according to it's position."

	| d |
	self contents.	"Force contents creation."
	item value add: anItem item beforeIndex: anIndex.
	anItem parentNode: self.
	^ contents add: anItem beforeIndex: anIndex! !

!ABPackageCategoryNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/14/2012 23:17'!
asString
	^ item key! !

!ABPackageCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 8/1/2012 10:42'!
childNodeClassFromItem: anItem
	(anItem isKindOf: Association )
		ifTrue: [ ^ self class ].
	^ ABPackageNode! !

!ABPackageCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/14/2012 23:18'!
childrenItems
	^ item value! !

!ABPackageCategoryNode methodsFor: 'commands' stamp: 'ThierryGoubier 8/5/2012 22:55'!
clipboardTreeCommands
	<treeAreaCommand>
	^ {ABRemovePackageCategoryCommand.
	ABCreatePackageCategoryCommand}! !

!ABPackageCategoryNode methodsFor: 'drag and drop' stamp: 'ThierryGoubier 8/29/2012 23:45'!
dropItem: anItem
	"Ok. If we receive a category or a package we either add it to its
	children (if it has no children ?) or add it just before this item?"

	((anItem isKindOf: ABPackageCategoryNode) or: [ anItem isKindOf: ABPackageNode ])
		ifFalse: [ ^ self ].
	anItem parentNode isNil
		ifTrue: [ AltBrowser rootItems contents remove: anItem ]
		ifFalse: [ anItem parentNode removeChildItem: anItem ].
	AltBrowser announcer announce: (AltNodeRemoved node: anItem).
	self childrenItems isEmpty
		ifTrue: [ self addChildItem: anItem ]
		ifFalse: [ 
			self parentNode isNil
				ifTrue: [ 
					AltBrowser rootItems contents add: anItem before: self. anItem parentNode: nil ]
				ifFalse: [ self parentNode addChildItem: anItem beforeIndex: (self parentNode contents indexOf: self) ] ].
	AltBrowser announcer announce: (AltNodeAdded node: anItem)! !

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 21:24'!
removeChildItem: anItem
	"Remove a child."

	| d |
	item value remove: anItem item.
	contents isNil
		ifTrue: [ ^ nil ]
		ifFalse: [ ^ contents remove: anItem ]! !

ABAbstractNode subclass: #ABPackageNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageNode commentStamp: '<historical>' prior: 0!
An ABPackageNode represent a package and all classes matching it (categories).
!

ABPackageNode subclass: #ABPackageExtensionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageExtensionNode commentStamp: '<historical>' prior: 0!
An ABPackageExtensionNode is for the extensions part of a package.!

!ABPackageExtensionNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/17/2012 13:31'!
asString
	^ '*extensions*'! !

!ABPackageExtensionNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:38'!
childNodeClassFromItem: anItem
	^ ABClassExtensionNode! !

!ABPackageExtensionNode methodsFor: 'private' stamp: 'ThierryGoubier 8/27/2012 00:19'!
childrenItems
	^ self item extendedClasses! !

!ABPackageNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/15/2012 07:49'!
asString
	^ self item packageName! !

!ABPackageNode methodsFor: 'private' stamp: 'ThierryGoubier 8/27/2012 17:24'!
childNodeClassFromItem: anItem
	((anItem isKindOf: Class) or: [ anItem isKindOf: Metaclass ])
		ifTrue: [ ^ ABClassNode ].
	(anItem isKindOf: RPackage)
		ifTrue: [ ^ ABPackageExtensionNode ].
	^ ABClassCategoryNode! !

!ABPackageNode methodsFor: 'private' stamp: 'ThierryGoubier 8/28/2012 15:09'!
childrenItems
	"Extract from the organisation the various classes."

	| c o |
	c := self item tags.
	o := OrderedCollection new.
	o add: self item.
	c size = 1
		ifTrue: [ c do: [ :e | (self item classesForClassTag: e) do: [ :class | o add: (Smalltalk globals at: class) ] ] ]
		ifFalse: [ o addAll: (c" reject: [ :e | Smalltalk organization isEmptyCategoryNamed: e ]") ].
	^ o! !

!ABPackageNode methodsFor: 'drag and drop' stamp: 'ThierryGoubier 8/29/2012 23:49'!
dropItem: anItem
	"Ok. If we receive a category, then we add the category at the end of the parent node. It amounts to removing a node from the root items and adding it here."

	((anItem isKindOf: ABPackageCategoryNode) or: [ anItem isKindOf: ABPackageNode ])
		ifFalse: [ ^ self ].
	anItem parentNode isNil
		ifTrue: [ AltBrowser rootItems contents remove: anItem ]
		ifFalse: [ anItem parentNode removeChildItem: anItem ].
	AltBrowser announcer announce: (AltNodeRemoved node: anItem).
	self parentNode addChildItem: anItem beforeIndex: (parentNode contents indexOf: self).
	AltBrowser announcer announce: (AltNodeAdded node: anItem)! !

!ABPackageNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 21:17'!
findPackage: aMCWorkingCopy
	"As a package extension node inherit and is set below, then it is never reached."

	^ item == aMCWorkingCopy
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABPackageNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/27/2012 17:20'!
sourceCode
	"Was description before, but this was lost in the PackageInfo to RPackage transition."
	
	^ self item name! !

!ABPackageNode methodsFor: 'commands' stamp: 'ThierryGoubier 8/30/2012 16:34'!
test
	^ self! !

TreeModel subclass: #ABTreeModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTreeModel commentStamp: '<historical>' prior: 0!
An ABTreeModel is an extension of TreeModel to handle drag and drop.

My problem with the drag and drop is that I loose the wrapper object (and therefore the logic) which is needed when the drag starts. I can :
* add another level of wrapper object below the list objects.
* have only one type of list wrapper object and my own wrappers.
* subclass the PluggableTreeMorph.!

!ABTreeModel class methodsFor: 'specs' stamp: 'ThierryGoubier 7/31/2012 10:41'!
defaultSpec
	<spec>
	
	^ {#TreeSpec.
	    		#model:.							#model.
			#getLabelSelector:.				#wrapItem:.
			#getRootsSelector:.				#getRoots.
			#getChildrenSelector:.				#getChildrenFor:.
			#setSelectedSelector:.			#setSelectedItem:.
			#color:.							Color white.
			#getMenuSelector:.				#getMenu:.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}.
			#dropItemSelector:.				#dropItem:on:.
			#dragItemSelector:.				#dragItem:.
			#wantsDropSelector:.				#wantsDropItem:.}! !

!ABTreeModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 7/31/2012 11:07'!
dragItem: anItem
	^ anItem! !

!ABTreeModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 8/1/2012 21:22'!
dropItem: anItem on: aTarget
	"This is called. Yes, really."

	aTarget notNil
		ifTrue: [ aTarget dropItem: anItem ]! !

!ABTreeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 8/30/2012 16:30'!
rebuildNode: aNode
	| r result |
	rootsHolder contents detect: [ :rootItem | (result := rootItem findItem: aNode) notNil ].
	result notNil
		ifTrue: [ 
			result rebuildContents.
			r := (self widget scroller submorphs at: 1) findItem: result.
			r notNil
				ifTrue: [ r rebuildChildren ] ]! !

!ABTreeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 22:46'!
rootsAdd: aNode
	"Algorithm: look for the parent of the node. If this parent node is found, then add a node equivalent to the contents if they do exist."

	| pNode |
	(pNode := aNode parentNode) isNil
		ifTrue: [
			self assert: (aNode isKindOf: ABTreeNodeModel) not.
			rootsHolder add: (ABTreeNodeModel with: aNode model: aNode) beforeIndex: (AltBrowser rootItems contents indexOf: aNode) ]
		ifFalse: [ 
			rootsHolder contents
				do: [ :e | 
					| r |
					(r := e findItem: pNode) notNil
						ifTrue: [ 
							| n wNode |
							r contentsNotNil
								ifTrue: [ wNode := r addChild: aNode ].
							(n := (self widget scroller submorphs at: 1) findItem: r) notNil
								ifTrue: [ 
									n hasContents
										ifTrue: [ n addItem: (ABTreeNodeModel with: wNode model: wNode) ] ].
							^ self ] ] ]! !

!ABTreeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 22:23'!
rootsRemove: aNode
	"Beware : a node is not an instance of ABTreeNodeModel"

	self assert: (aNode isTreeNodeModel not).
	rootsHolder contents
		do: [ :e | 
			| r |
			(r := e findItem: aNode) notNil
				ifTrue: [ 
					r == e
						ifTrue: [ rootsHolder remove: r ifAbsent: [ nil ] ]
						ifFalse: [ r parentNode contents remove: r.
						((self widget scroller submorphs at: 1) removeItem: r)] ] ]! !

!ABTreeModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 7/31/2012 10:49'!
wantsDropItem: anItem
	"TODO: Filter out to avoid drag and drop of different objet types. Here, we don't know what the target may be so filtering is limited."
	
	^ true
	! !

Announcement subclass: #AltNodeAnnouncement
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltNodeAnnouncement commentStamp: '<historical>' prior: 0!
The abstract node announcement.

Beware, unexpected behavior!! Announcement matching for actions matches also subclasses (what a strange idea?).

Buggy previous code: AltNodeRemoved has AltNodeAdded as superclass; on AltNodeRemoved both AltNodeAdded and AltNodeRemoved would match and trigger actions.
!

AltNodeAnnouncement subclass: #AltNodeAdded
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltNodeAdded commentStamp: '<historical>' prior: 0!
This announce an added node in the tree.
!

!AltNodeAnnouncement class methodsFor: 'instance creation' stamp: 'ThierryGoubier 8/29/2012 22:33'!
node: aNode
	^ self new node: aNode! !

!AltNodeAnnouncement methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 22:28'!
node
	^ node! !

!AltNodeAnnouncement methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 22:28'!
node: anObject
	node := anObject! !

AltNodeAnnouncement subclass: #AltNodeRebuilt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltNodeRebuilt commentStamp: '<historical>' prior: 0!
This announcement mark a node rebuilt.!

AltNodeAnnouncement subclass: #AltNodeRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltNodeRemoved commentStamp: '<historical>' prior: 0!
This class mark the removal of a node.!

CustomHelp subclass: #AltBrowserHelp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltBrowserHelp commentStamp: '<historical>' prior: 0!
This is the help for the AltBrowser. Tries to hold some documentation about what is going on.
!

TextModel subclass: #ABTextModel
	instanceVariableNames: 'menuBlockHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTextModel commentStamp: '<historical>' prior: 0!
An ABTextModel is an extension to TextModel allowing the override of the menu.
!

!ABTextModel class methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 7/17/2012 22:42'!
defaultSpec
	<spec>
	
	^ SpecLayout text
			send: #on:text:accept:readSelection:menu: withArguments: #(model getText accept:notifying: readSelection getMenu:);
			send: #classOrMetaClass: withArguments: {#(model behavior)};
			send: #enabled: withArguments: {#(model enabled)};
			send: #eventHandler: withArguments: {#(EventHandler on:send:to: keyStroke keyStroke:fromMorph: model)};
			send: #vSpaceFill;
			send: #hSpaceFill;
			yourself! !

!ABTextModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 22:39'!
getMenu: aMenu

	menuBlockHolder contents cull: aMenu cull: self.
	^ aMenu! !

!ABTextModel methodsFor: 'initialization' stamp: 'ThierryGoubier 7/17/2012 22:40'!
initialize
	"Initialization code for TreeModel"

	super initialize.

	menuBlockHolder := [] asValueHolder.! !

!ABTextModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 22:39'!
menu: aBlock
	"Set the block used to generate the tree menu"

	menuBlockHolder contents: aBlock! !

KMDispatcher subclass: #AltKMDispatcher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltKMDispatcher commentStamp: '<historical>' prior: 0!
Here only to remove a bad behavior in KMDispatcher. See how the standard KMDispatcher is replaced by this one in AltBrowser>>updateTextKeymaps.

Also remove the staticTargets targets, so that we may have a text editor without any legacy shortcuts.
!

!AltKMDispatcher methodsFor: 'dispatching' stamp: 'ThierryGoubier 8/7/2012 20:29'!
keymapObservers
	| o |
	(o := OrderedCollection with: self perInstanceTarget)
		addAll: self targets.
	^ o! !

MorphTreeNodeModel subclass: #ABTreeNodeModel
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTreeNodeModel commentStamp: '<historical>' prior: 0!
An ABTreeNodeModel represent a node for the tree. It reroutes most of it's request to the underlying item.

It caches its contents however.!

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 17:19'!
addChild: anItem
	"Add a child, but do it according to it's position."

	| d |
	d := self childrenItems indexOf: anItem.
	d < 1 ifTrue: [^ self].
	^ contents add: (self childNodeFromItem: anItem) beforeIndex: d! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/25/2012 23:23'!
buildTextMenu: aMenu on: aTarget with: aRequestor
	^ self item buildTextMenu: aMenu on: aTarget with: aRequestor! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/30/2012 16:54'!
buildTreeMenu: aMenu on: aTarget with: aRequestor
	^ self item buildTreeMenu: aMenu on: aTarget with: aRequestor! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:33'!
childrenItems
	^ item contents! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:26'!
contents
	contents isNil
		ifTrue: [ contents := super contents ].
	^ contents! !

!ABTreeNodeModel methodsFor: 'testing' stamp: 'ThierryGoubier 7/27/2012 21:42'!
contentsNotNil
	^ contents notNil! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/26/2012 23:16'!
findItem: anItem
	"searching for an item, returns a node."

	self item == anItem
		ifTrue: [ ^ self ].
	contents notNil
		ifTrue: [ 
			contents
				do: [ :e | 
					| s |
					(s := e findItem: anItem) notNil
						ifTrue: [ ^ s ] ] ].
	^ nil! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 23:24'!
isStyled
	^ item isStyled! !

!ABTreeNodeModel methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 14:44'!
isTreeNodeModel
	^ true! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 09:10'!
rebuildChildren
	"Force a rebuild of the children, including on the item."
	
	item rebuildContents.
	contents := nil! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 8/30/2012 16:29'!
rebuildContents
	"Force a rebuild of the contents."
	
	contents := nil! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:27'!
sourceClass
	^ item sourceClass! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:27'!
sourceCode
	^ item sourceCode! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/31/2012 10:35'!
wantsDroppedObject: anotherItem
	"So far, this is unused. The wrapper never connect to that here. What is needed is to customize the PluggableTreeMorph, which is the model behind all elements of the tree."

	^ self item wantsDroppedObject: anotherItem! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/30/2012 17:36'!
addItem: aNode
	"Add a submorph for that item. Sometime, the item may already be there; it depends on how
	the code goes through this. Modified to note where the node is to be put (ordering)."

	self refreshExpandedState.
	container adjustSubmorphPositions.

	"
	| newMorph lc nc |
	(self findItem: aNode item) ifNotNil: [ ^ self ].
	aNode parentNode notNil
		ifTrue: [ 
			| i j |
			i := aNode parentNode contents indexOf: aNode.
			j := 1.
			[ j < i ]
				whileTrue: [ 
					lc isNil
						ifTrue: [ lc := firstChild ]
						ifFalse: [ lc := lc nextSibling ] ] ]
		ifFalse: [ lc := self lastChild ].
	lc isNil
		ifFalse: [ nc := lc nextSibling ].
	newMorph := container indentingItemClass basicNew
		initWithContents: aNode
		prior: lc
		forList: container
		indentLevel: lc indentLevel.
	lc isNil
		ifTrue: [ firstChild := newMorph ]
		ifFalse: [ lc nextSibling: newMorph ].
	nc isNil
		ifFalse: [ newMorph nextSibling: nc ].
	self isExpanded
		ifTrue: [ 
			container scroller addAllMorphs: {newMorph} after: lc.
			container adjustSubmorphPositions ]"! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/27/2012 14:35'!
findItem: aNode
	| found |
	self
		withSiblingsDo: [ :each | 
			each complexContents item == aNode
				ifTrue: [ ^ each ] ].
	self
		withSiblingsDo: [ :each | 
			each firstChild notNil
				ifTrue: [ 
					found := each firstChild findItem: aNode.
					found notNil
						ifTrue: [ ^ found ] ] ].
	^ nil! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/27/2012 14:38'!
hasContents
	^ firstChild notNil! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 8/5/2012 08:55'!
rebuildChildren
	"Force a rebuild of the childrens. May be triggered by going from no children to has children."

	self refreshExpandedState.
	container adjustSubmorphPositions! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/27/2012 17:03'!
removeItem: aNode
	| found |
	found := self findItem: aNode.
	found isNil
		ifFalse: [ 
			found recursiveDelete.
			container adjustSubmorphPositions ].
	^ found! !

TestCase subclass: #AltBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Tests'!

!AltBrowserTest methodsFor: 'tests' stamp: 'ThierryGoubier 8/26/2012 23:57'!
testRootItems
	self assert: (AltBrowser getRootItems notEmpty)! !

!AltBrowserTest methodsFor: 'tests' stamp: 'ThierryGoubier 8/27/2012 00:06'!
testSpawn2RootItems
	AltBrowser getRootItems do: [:e | e contents do: [:f | f contents]]! !

!AltBrowserTest methodsFor: 'tests' stamp: 'ThierryGoubier 8/27/2012 00:04'!
testSpawnRootItems
	AltBrowser getRootItems do: [:e | e contents]! !

Object subclass: #ABCommand
	instanceVariableNames: 'target requestor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCommand commentStamp: '<historical>' prior: 0!
An ABCommand is in charge of preparing and executing commands on the Alt Browser.

Instance Variables
	requestor:	<Object>	The object sending the command?
	target:		<Object>	The target (where the command is applied)!

ABCommand subclass: #ABAcceptTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABAcceptTextCommand commentStamp: '<historical>' prior: 0!
An ABAcceptTextCommand is about accepting the text.
!

!ABAcceptTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABAcceptTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABAcceptTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 14:20'!
execute
	target widget accept.
	requestor selectedItem item accept: target getText! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:39'!
group
	^ #zip! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallOkIcon! !

!ABAcceptTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:21'!
isActive
	^ true! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:50'!
keystroke
	^ $s ctrl ! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:09'!
label
	^ 'Accept'! !

ABCommand subclass: #ABAddProtocolCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABAddProtocolCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/29/2012 11:07'!
execute
	"Reuse code from Browser>>addCategory"

	"Present a choice of categories or prompt for a new category name and add it before the current selection, or at the end if no current selection"
	| labels reject lines oldIndex newName tClass sItem |
	requestor okToChange ifFalse: [^ self].
	labels := OrderedCollection new.
	sItem := requestor selectedItem.
	tClass := requestor selectedItem sourceClass.
	reject := Set new.
	reject
		addAll: tClass organization categories;
		add: ClassOrganizer nullCategory;
		add: ClassOrganizer default.
	lines := OrderedCollection new.
	tClass allSuperclasses do: [:cls | | cats |
		cls = Object ifFalse: [
			cats := cls organization categories reject:
				 [:cat | reject includes: cat].
			cats isEmpty ifFalse: [
				lines add: labels size.
				labels addAll: cats asSortedCollection.
				reject addAll: cats ] ] ].
	
	newName := UIManager default chooseOrRequestFrom: labels lines: lines title: 'Add Category'.
	newName ifNil: [ ^self ].
	newName := newName asSymbol.
	
	tClass
		addCategory: newName
		before: ((sItem item isKindOf: ABMethodCategoryNode)
				ifTrue: [nil]
				ifFalse: [sItem item item]).
	
	requestor selectedItem: (sItem parentNode findItem: (sItem parentNode item findMethodCategory: newName)).
! !

!ABAddProtocolCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/29/2012 10:39'!
isActive
	^ true! !

!ABAddProtocolCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 10:39'!
label
	^ 'Add protocol'! !

ABCommand subclass: #ABCancelTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCancelTextCommand commentStamp: '<historical>' prior: 0!
An ABCancelTextCommand is about cancelling the last action.!

!ABCancelTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCancelTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCancelTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/2/2012 13:34'!
execute
	target widget cancel! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:39'!
group
	^ #zip! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCancelIcon! !

!ABCancelTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:51'!
keystroke
	^ $l ctrl! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:11'!
label
	^ 'Cancel'! !

!ABCommand class methodsFor: 'instance creation' stamp: 'cwp 10/14/2006 23:29'!
on: target for: requestor
	^ self new setTarget: target requestor: requestor! !

!ABCommand class methodsFor: 'testing' stamp: 'cwp 10/15/2006 00:58'!
takesNodes
	^ true! !

!ABCommand class methodsFor: 'testing' stamp: 'cwp 10/15/2006 00:58'!
takesText
	^ false! !

!ABCommand methodsFor: 'user interface' stamp: 'ThierryGoubier 7/25/2012 23:27'!
addItemToMenu: aMenu
	self isActive
		ifTrue: [ 
			(aMenu add: self label target: self selector: #execute)
				enablementSelector: #isEnabled;
				icon:
						(self icon notNil
								ifTrue: [ UITheme current perform: self icon ]
								ifFalse: [ nil ]);
				keyText: self keyString ]! !

!ABCommand methodsFor: 'user interface' stamp: 'ThierryGoubier 8/5/2012 22:27'!
buildShortcut: aKMDispatcher
	"Add a shortcut to the keymap. Not sure about what the category should be."
	
	aKMDispatcher on: self keystroke do: [self execute] ! !

!ABCommand methodsFor: 'accessing' stamp: 'lr 4/25/2007 09:27'!
buttonLabel
	^ self label! !

!ABCommand methodsFor: 'execution' stamp: 'cwp 10/15/2006 14:12'!
execute
	! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 21:44'!
group
	^ #general! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 18:45'!
icon
	^ nil! !

!ABCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 22:05'!
isActive
	^ false! !

!ABCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 19:31'!
isEnabled
	^ true! !

!ABCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/7/2012 22:49'!
keyString
	| c |
	^ (c := self keystroke) isNil
		ifFalse: [ c asString copyWithout: Character space ]
		ifTrue: [ nil ]! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
keystroke
	^ nil
! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
label
	self subclassResponsibility
! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 10/14/2006 23:39'!
on: anObject for: aRequestor
	^ self class on: anObject for: aRequestor! !

!ABCommand methodsFor: 'user interface' stamp: 'dc 1/3/2008 10:11'!
order
	^ self label! !

!ABCommand methodsFor: 'user interface' stamp: 'cwp 10/15/2006 12:37'!
perform: aSelector orSendTo: anObject
	self perform: aSelector! !

!ABCommand methodsFor: 'user interface' stamp: 'lr 11/25/2010 23:45'!
select: aNode with: anAnnouncer
	aNode announceSelectionWith: anAnnouncer! !

!ABCommand methodsFor: 'initialize-release' stamp: 'lr 7/3/2009 22:27'!
setTarget: anObject requestor: aRequestor 
	target := anObject.
	requestor := aRequestor! !

!ABCommand methodsFor: 'testing' stamp: 'lr 2/7/2010 12:50'!
takesNodes
	^ self class takesNodes! !

!ABCommand methodsFor: 'testing' stamp: 'lr 2/7/2010 12:50'!
takesText
	^ self class takesText! !

!ABCommand methodsFor: 'testing' stamp: 'cwp 9/27/2006 22:53'!
wantsButton
	"Put this here for compatibility. Eventually this will be obsolete"

	^ false
! !

!ABCommand methodsFor: 'testing' stamp: 'lr 8/8/2010 10:47'!
wantsMenu
	"Put this here for compatibility. Eventually this will be obsolete"

	^ true
! !

ABCommand subclass: #ABCopyTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCopyTextCommand commentStamp: '<historical>' prior: 0!
This class does a copy on the text area.!

!ABCopyTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCopyTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCopyTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/2/2012 13:35'!
execute
	target widget copySelection! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:37'!
group
	^ #clipboard! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCopyIcon! !

!ABCopyTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABCopyTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:46'!
isEnabled
	^ target widget trueSelection isEmpty not! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:51'!
keystroke
	^ $c ctrl! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:11'!
label
	^ 'Copy'! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '2'! !

ABCommand subclass: #ABCreatePackageCategoryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCreatePackageCategoryCommand commentStamp: '<historical>' prior: 0!
This class prompts for a name and creates a category at top level.
!

!ABCreatePackageCategoryCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/29/2012 22:45'!
execute
	"Ask for a category name and create it."

	| name |
	name := UIManager default request: 'New Category name?'.
	name notNil
		ifTrue: [ 
			| item |
			item := ABPackageCategoryNode with: name -> OrderedCollection new.
			requestor selectedItem notNil
				ifTrue: [ AltBrowser rootItems contents add: item afterIndex: (AltBrowser rootItems contents indexOf: requestor selectedItem item) ]
				ifFalse: [ AltBrowser rootItems contents add: item ].
			AltBrowser announcer announce: (AltNodeAdded node: item) ]! !

!ABCreatePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 15:52'!
icon
	^ nil! !

!ABCreatePackageCategoryCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 15:54'!
isActive
	^ true! !

!ABCreatePackageCategoryCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 15:59'!
isEnabled
	^ requestor selectedItem isNil or: [ requestor selectedItem item isKindOf: ABPackageCategoryNode ]! !

!ABCreatePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 15:58'!
label
	^'Create '! !

ABCommand subclass: #ABCutTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCutTextCommand commentStamp: '<historical>' prior: 0!
This class does a cut in the text area.
!

!ABCutTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCutTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCutTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/2/2012 13:34'!
execute
	target widget cut! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCutIcon! !

!ABCutTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABCutTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:49'!
isEnabled
	^ target widget trueSelection isEmpty not! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 15:27'!
keystroke
	^ $x ctrl! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:12'!
label
	^ 'Cut'! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '1'! !

ABCommand subclass: #ABDoItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABDoItTextCommand commentStamp: '<historical>' prior: 0!
ABDoItTextCommand : execute the code selected
!

ABDoItTextCommand subclass: #ABDebugItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABDebugItTextCommand commentStamp: '<historical>' prior: 0!
This class does a debugIt on the selection in the text area.
!

!ABDebugItTextCommand class methodsFor: 'testing' stamp: 'lr 8/27/2011 10:02'!
takesNodes
	^ false! !

!ABDebugItTextCommand class methodsFor: 'testing' stamp: 'lr 8/27/2011 10:02'!
takesText
	^ true! !

!ABDebugItTextCommand methodsFor: 'execution' stamp: 'lr 8/27/2011 09:58'!
execute
	target debugIt! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallDebugIcon! !

!ABDebugItTextCommand methodsFor: 'testing' stamp: 'lr 8/27/2011 09:58'!
isActive
	^ true! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:52'!
keystroke
	^ $d command shift! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'lr 8/27/2011 09:58'!
label
	^ 'Debug it'! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'lr 8/27/2011 10:00'!
order
	^ '5'! !

!ABDoItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABDoItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABDoItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 10:41'!
execute
	"Treat the current selection as an expression; evaluate it."

	| result rcvr ctxt |
	(requestor selectedItem item respondsTo: #doItReceiver)
		ifTrue: [ 
			rcvr := requestor selectedItem item doItReceiver.
			ctxt := requestor selectedItem item doItContext ]
		ifFalse: [ rcvr := ctxt := nil ].
	result := [ 
	rcvr class evaluatorClass new
		evaluate: (ReadStream on: target widget trueSelection)
		in: ctxt
		to: rcvr
		notifying: target widget
		ifFail: [ 
			target widget flash.
			^ nil ]
		logged: true	"don't set it to self.. The receiver can be destroyed as a result of evaluateSelection" ]
		on: OutOfScopeNotification
		do: [ :ex | ex resume: true ].
	^ result! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:55'!
group
	^ #smalltalk! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallDoItIcon! !

!ABDoItTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/25/2012 23:25'!
isActive
	^ target readSelection notNil! !

!ABDoItTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 10:40'!
isEnabled
	^ target widget trueSelection notEmpty! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:51'!
keystroke
	^ $d command! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:55'!
label
	^ 'Do it'! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:54'!
order
	^ '1'! !

ABDoItTextCommand subclass: #ABExploreItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABExploreItTextCommand commentStamp: '<historical>' prior: 0!
This class does an exploreIt (evaluate and explore) on the contents selected in the text area.
!

!ABExploreItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 15:08'!
takesNodes
	^ false! !

!ABExploreItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 15:08'!
takesText
	^ true! !

!ABExploreItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 13:47'!
execute
	super execute explore! !

!ABExploreItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 15:05'!
isActive
	^ true! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:52'!
keystroke
	^ $I command! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 15:05'!
label
	^ 'Explore it'! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 15:05'!
order
	^ '4'! !

ABDoItTextCommand subclass: #ABInspectItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABInspectItTextCommand commentStamp: '<historical>' prior: 0!
This class does an inspectIt (evaluate and inspect the result) on the selection in the text area.
!

!ABInspectItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABInspectItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABInspectItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 13:48'!
execute
	super execute inspect! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallInspectItIcon! !

!ABInspectItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
isActive
	^ true! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:52'!
keystroke
	^ $i command! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
label
	^ 'Inspect it'! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
order
	^ '3'! !

ABDoItTextCommand subclass: #ABPrintItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABPrintItTextCommand commentStamp: '<historical>' prior: 0!
This command does a printIt (evaluate and print the result) on the selection in the text area.!

!ABPrintItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABPrintItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABPrintItTextCommand methodsFor: 'execution' stamp: 'lr 3/6/2011 15:13'!
execute
	target printIt! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallPrintIcon! !

!ABPrintItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
isActive
	^ true! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:53'!
keystroke
	^ $p command! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
label
	^ 'Print it'! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
order
	^ '2'! !

ABCommand subclass: #ABExploreItemCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABExploreItemCommand commentStamp: '<historical>' prior: 0!
An ABExploreItemCommand does an explore of the selected item in the tree pane.
!

!ABExploreItemCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/30/2012 16:47'!
execute
	requestor selectedItem item explore! !

!ABExploreItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/30/2012 16:51'!
icon
	^ #smallInspectItIcon! !

!ABExploreItemCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/30/2012 16:48'!
isActive
	^ true! !

!ABExploreItemCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/30/2012 16:48'!
isEnabled
	^ requestor selectedItem notNil! !

!ABExploreItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 22:07'!
keystroke
	^ $l command! !

!ABExploreItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/30/2012 16:49'!
label
	^ 'ExploreIt'! !

ABCommand subclass: #ABFindClassCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABFindClassCommand commentStamp: '<historical>' prior: 0!
This command does a find class. Use an openPath approach.!

!ABFindClassCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/7/2012 22:56'!
execute
	| aClass |
	aClass := SearchFacade classSearch chooseFromOwner: requestor window window.
	aClass explore
! !

!ABFindClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/7/2012 22:51'!
icon
	^ nil! !

!ABFindClassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/7/2012 22:50'!
isActive
	^ true! !

!ABFindClassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/7/2012 22:50'!
isEnabled
	^ true! !

!ABFindClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/7/2012 22:52'!
keystroke
	^ $f command, $c command! !

!ABFindClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/7/2012 22:50'!
label
	^ 'Find Class'! !

ABCommand subclass: #ABPasteTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABPasteTextCommand commentStamp: '<historical>' prior: 0!
An ABPasteTextCommand : Paste text from Clipboard. Directly target the editor; neither the model nor the widget have methods for handling that.
!

!ABPasteTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABPasteTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABPasteTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:03'!
execute
	target widget textMorph editor replace: target widget trueSelectionInterval with: Clipboard clipboardText and: [  ]! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:27'!
group
	^ #clipboard! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallPasteIcon! !

!ABPasteTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABPasteTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:52'!
isEnabled
	^ Clipboard clipboardText notEmpty! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:53'!
keystroke
	^ $v ctrl! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:14'!
label
	^ 'Paste'! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '3'! !

ABCommand subclass: #ABRebuildItemCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRebuildItemCommand commentStamp: '<historical>' prior: 0!
This class forces a rebuild of an item in the tree.
!

!ABRebuildItemCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/30/2012 16:05'!
execute
	"requestor selectedItem rebuildChildren."
	AltBrowser aNodeToRebuilt: requestor selectedItem item! !

!ABRebuildItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 09:08'!
icon
	^ #smallDebugIcon! !

!ABRebuildItemCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/5/2012 09:00'!
isActive
	^ true! !

!ABRebuildItemCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/5/2012 09:01'!
isEnabled
	^ requestor selectedItem notNil! !

!ABRebuildItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 08:59'!
keystroke
	^ nil! !

!ABRebuildItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 09:00'!
label
	^ 'Rebuild It'! !

ABCommand subclass: #ABRedoTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRedoTextCommand commentStamp: '<historical>' prior: 0!
An ABRedoTextCommand calls the text editor redo command.
!

!ABRedoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABRedoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABRedoTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:07'!
execute
	target widget textMorph editor redo! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:38'!
group
	^ #adapting! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallRedoIcon! !

!ABRedoTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:54'!
keystroke
	^ $j ctrl! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:20'!
label
	^ 'Redo'! !

ABCommand subclass: #ABRemovePackageCategoryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRemovePackageCategoryCommand commentStamp: '<historical>' prior: 0!
This class does a remove item on the selected item (package category : anything else requires specific system commands.)!

!ABRemovePackageCategoryCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/29/2012 17:19'!
execute
	"Remove the package category."

	| item |
	item := requestor selectedItem item.
	item parentNode notNil
		ifTrue: [ item parentNode removeChildItem: item ]
		ifFalse: [ AltBrowser rootItems contents remove: item ].
	AltBrowser announcer announce: (AltNodeRemoved node: item)! !

!ABRemovePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 15:26'!
icon
	^ #smallCutIcon! !

!ABRemovePackageCategoryCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 15:27'!
isActive
	^ true! !

!ABRemovePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 15:27'!
label
	^ 'Cut'! !

ABCommand subclass: #ABRemoveProtocolCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRemoveProtocolCommand methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 8/30/2012 16:17'!
execute
	| tClass tItem |
	
	tClass := requestor selectedItem sourceClass.
	tItem := requestor selectedItem.
	tClass removeCategory: tItem item item! !

!ABRemoveProtocolCommand methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 8/30/2012 16:13'!
isActive
	^ true! !

!ABRemoveProtocolCommand methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 8/30/2012 16:14'!
label
	^ 'Cut'! !

ABCommand subclass: #ABUndoTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABUndoTextCommand commentStamp: '<historical>' prior: 0!
An ABUndoTextCommand calls the text editor undo command.!

!ABUndoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABUndoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABUndoTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:07'!
execute
	target widget textMorph editor undo! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:38'!
group
	^ #adapting! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:19'!
icon
	^ #smallUndoIcon! !

!ABUndoTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:54'!
keystroke
	^ $z ctrl! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:20'!
label
	^ 'Undo'! !

!PluggableTextMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/26/2012 07:48'!
trueSelection
	"Return the selection text as remembered by the PluggableTextMoprh, even if not in focus."

	^ textMorph editor text copyFrom: selectionInterval first to: selectionInterval last! !

!PluggableTextMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/25/2012 23:58'!
trueSelectionInterval
	^ selectionInterval! !
AltBrowser initialize!
